/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.finance.wrapper;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.common.cache.DictBizCache;
import org.springblade.common.cache.DictCache;
import org.springblade.common.enums.DictBizEnum;
import org.springblade.common.enums.DictEnum;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.entity.OrderGoods;
import org.springblade.modules.finance.service.IOrderGoodsService;
import org.springblade.modules.finance.vo.OrderVO;

import java.util.List;
import java.util.Objects;

/**
 * 包装类,返回视图层所需的字段
 *
 * @author Chill
 */
public class OrderWrapper extends BaseEntityWrapper<Order, OrderVO> {

	public static OrderWrapper build() {
		return new OrderWrapper();
	}

	private static IOrderGoodsService orderGoodsService;

	static {
		orderGoodsService = SpringUtil.getBean(IOrderGoodsService.class);
	}

	@Override
	public OrderVO entityVO(Order entity) {
		OrderVO vo = Objects.requireNonNull(BeanUtil.copy(entity, OrderVO.class));
		vo.setDistributeModeDisplayName(DictBizCache.getValue(DictBizEnum.DISTRIBUTE_MODE,vo.getDistributeMode()));
		vo.setPayFlagDisplayName(DictBizCache.getValue(DictBizEnum.PAY_FLAY,vo.getPayFlag()));
		vo.setOrderTypeDisplayName(DictBizCache.getValue(DictBizEnum.ORDER_TYPE,vo.getOrderType()));
		vo.setIsRefundDisplayName(DictCache.getValue(DictEnum.YES_NO,vo.getIsRefund()));
		List<OrderGoods> list = orderGoodsService.list(Wrappers.<OrderGoods>lambdaQuery().eq(OrderGoods::getOutTradeNo,entity.getOutTradeNo()));
		if (list.size() > 0){
			vo.setOrderGoods(list);
		}
		return vo;
	}

}
