/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.shopping.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.shopping.entity.Goods;
import org.springblade.modules.shopping.service.IGoodsService;
import org.springblade.modules.shopping.wrapper.SpecItemsWrapper;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.shopping.entity.SpecItems;
import org.springblade.modules.shopping.vo.SpecItemsVO;
import org.springblade.modules.shopping.service.ISpecItemsService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 商品规格值表 控制器
 *
 * @author BladeX
 * @since 2022-02-13
 */
@RestController
@AllArgsConstructor
@RequestMapping("shopping/specitems")
@Api(value = "商品规格值表", tags = "商品规格值表接口")
public class SpecItemsController extends BladeController {

	private final ISpecItemsService specItemsService;
	private final IGoodsService goodsService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入specItems")
	public R<SpecItems> detail(SpecItems specItems) {
		SpecItems detail = specItemsService.getOne(Condition.getQueryWrapper(specItems));
		return R.data(detail);
	}

	/**
	 * 分页 商品规格值表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入specItems")
	public R<IPage<SpecItemsVO>> list(SpecItems specItems, Query query) {
		IPage<SpecItems> pages = specItemsService.page(Condition.getPage(query), Condition.getQueryWrapper(specItems));
		return R.data(SpecItemsWrapper.build().pageVO(pages));
	}

	/**
	 * 自定义分页 商品规格值表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入specItems")
	public R<IPage<SpecItemsVO>> page(SpecItemsVO specItems, Query query) {
		IPage<SpecItemsVO> pages = specItemsService.selectSpecItemsPage(Condition.getPage(query), specItems);
		return R.data(pages);
	}

	/**
	 * 新增 商品规格值表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入specItems")
	public R save(@Valid @RequestBody SpecItems specItems) {
		return R.status(specItemsService.save(specItems));
	}

	/**
	 * 修改 商品规格值表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入specItems")
	public R update(@Valid @RequestBody SpecItems specItems) {
		return R.status(specItemsService.updateById(specItems));
	}

	/**
	 * 新增或修改 商品规格值表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入specItems")
	public R submit(@Valid @RequestBody SpecItemsVO specItems) {
		if (specItems.getProductNo() != null){
			Goods goods = goodsService.getOne(Wrappers.<Goods>lambdaQuery().eq(Goods::getProductNo, specItems.getProductNo()));
			specItems.setGoodsId(goods.getId());
			specItems.setProductNo(goods.getProductNo());
		}
		return R.status(specItemsService.saveOrUpdate(specItems));
	}


	/**
	 * 删除 商品规格值表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(specItemsService.removeByIds(Func.toLongList(ids)));
	}


}
